@@ -131,7 +131,7 @@ std::string mapLLVMTypeToOCLType(const Type *Ty, bool Signed, Type *PET) {
     return "float";
   if (Ty->isDoubleTy())
     return "double";
-  if (auto IntTy = dyn_cast<IntegerType>(Ty)) {
+  if (const auto *IntTy = dyn_cast<IntegerType>(Ty)) {
     std::string SignPrefix;
     std::string Stem;
     if (!Signed)
@@ -155,7 +155,7 @@ std::string mapLLVMTypeToOCLType(const Type *Ty, bool Signed, Type *PET) {
     }
     return SignPrefix + Stem;
   }
-  if (auto VecTy = dyn_cast<FixedVectorType>(Ty)) {
+  if (const auto *VecTy = dyn_cast<FixedVectorType>(Ty)) {
     Type *EleTy = VecTy->getElementType();
     unsigned Size = VecTy->getNumElements();
     std::stringstream Ss;
@@ -178,7 +178,7 @@ std::string mapLLVMTypeToOCLType(const Type *Ty, bool Signed, Type *PET) {
 }
 
 StructType *getOrCreateOpaqueStructType(Module *M, StringRef Name) {
-  auto OpaqueType = StructType::getTypeByName(M->getContext(), Name);
+  auto *OpaqueType = StructType::getTypeByName(M->getContext(), Name);
   if (!OpaqueType)
     OpaqueType = StructType::create(M->getContext(), Name);
   return OpaqueType;
@@ -224,7 +224,7 @@ bool isSPIRVStructType(llvm::Type *Ty, StringRef BaseTyName,
         std::string(kSPIRVTypeName::PrefixAndDelim) + BaseTyName.str();
     if (FullName != N)
       N = N + kSPIRVTypeName::Delimiter;
-    if (FullName.startswith(N)) {
+    if (FullName.starts_with(N)) {
       if (Postfix)
         *Postfix = FullName.drop_front(N.size());
       return true;
@@ -240,9 +240,9 @@ bool isSYCLHalfType(llvm::Type *Ty) {
     StringRef Name = ST->getName();
     if (!Name.consume_front("class."))
       return false;
-    if ((Name.startswith("sycl::") || Name.startswith("cl::sycl::") ||
-         Name.startswith("__sycl_internal::")) &&
-        Name.endswith("::half")) {
+    if ((Name.starts_with("sycl::") || Name.starts_with("cl::sycl::") ||
+         Name.starts_with("__sycl_internal::")) &&
+        Name.ends_with("::half")) {
       return true;
     }
   }
@@ -256,9 +256,9 @@ bool isSYCLBfloat16Type(llvm::Type *Ty) {
     StringRef Name = ST->getName();
     if (!Name.consume_front("class."))
       return false;
-    if ((Name.startswith("sycl::") || Name.startswith("cl::sycl::") ||
-         Name.startswith("__sycl_internal::")) &&
-        Name.endswith("::bfloat16")) {
+    if ((Name.starts_with("sycl::") || Name.starts_with("cl::sycl::") ||
+         Name.starts_with("__sycl_internal::")) &&
+        Name.ends_with("::bfloat16")) {
       return true;
     }
   }
@@ -286,7 +286,7 @@ Function *getOrCreateFunction(Module *M, Type *RetTy, ArrayRef<Type *> ArgTypes,
     report_fatal_error(llvm::Twine(SS.str()), false);
   }
   if (!F || F->getFunctionType() != FT) {
-    auto NewF =
+    auto *NewF =
         Function::Create(FT, GlobalValue::ExternalLinkage, MangledName, M);
     if (F && TakeName) {
       NewF->takeName(F);
@@ -327,28 +327,13 @@ Scope getArgAsScope(CallInst *CI, unsigned I) {
   return static_cast<Scope>(getArgAsInt(CI, I));
 }
 
-Decoration getArgAsDecoration(CallInst *CI, unsigned I) {
-  return static_cast<Decoration>(getArgAsInt(CI, I));
-}
-
-std::string decorateSPIRVFunction(const std::string &S) {
-  return std::string(kSPIRVName::Prefix) + S + kSPIRVName::Postfix;
-}
-
-StringRef undecorateSPIRVFunction(StringRef S) {
-  assert(S.find(kSPIRVName::Prefix) == 0);
-  const size_t Start = strlen(kSPIRVName::Prefix);
-  auto End = S.rfind(kSPIRVName::Postfix);
-  return S.substr(Start, End - Start);
-}
-
 std::string prefixSPIRVName(const std::string &S) {
   return std::string(kSPIRVName::Prefix) + S;
 }
 
 StringRef dePrefixSPIRVName(StringRef R, SmallVectorImpl<StringRef> &Postfix) {
   const size_t Start = strlen(kSPIRVName::Prefix);
-  if (!R.startswith(kSPIRVName::Prefix))
+  if (!R.starts_with(kSPIRVName::Prefix))
     return R;
   R = R.drop_front(Start);
   R.split(Postfix, "_", -1, false);
@@ -391,7 +376,7 @@ SPIRVDecorate *mapPostfixToDecorate(StringRef Postfix, SPIRVEntry *Target) {
   if (Postfix == kSPIRVPostfix::Sat)
     return new SPIRVDecorate(spv::DecorationSaturatedConversion, Target);
 
-  if (Postfix.startswith(kSPIRVPostfix::Rt))
+  if (Postfix.starts_with(kSPIRVPostfix::Rt))
     return new SPIRVDecorate(spv::DecorationFPRoundingMode, Target,
                              map<SPIRVFPRoundingModeKind>(Postfix.str()));
 
@@ -401,23 +386,11 @@ SPIRVDecorate *mapPostfixToDecorate(StringRef Postfix, SPIRVEntry *Target) {
 SPIRVValue *addDecorations(SPIRVValue *Target,
                            const SmallVectorImpl<std::string> &Decs) {
   for (auto &I : Decs)
-    if (auto Dec = mapPostfixToDecorate(I, Target))
+    if (auto *Dec = mapPostfixToDecorate(I, Target))
       Target->addDecorate(Dec);
   return Target;
 }
 
-std::string getPostfix(Decoration Dec, unsigned Value) {
-  switch (Dec) {
-  default:
-    llvm_unreachable("not implemented");
-    return "unknown";
-  case spv::DecorationSaturatedConversion:
-    return kSPIRVPostfix::Sat;
-  case spv::DecorationFPRoundingMode:
-    return rmap<std::string>(static_cast<SPIRVFPRoundingModeKind>(Value));
-  }
-}
-
 std::string getPostfixForReturnType(CallInst *CI, bool IsSigned) {
   return getPostfixForReturnType(CI->getType(), IsSigned);
 }
@@ -431,7 +404,7 @@ std::string getPostfixForReturnType(const Type *PRetTy, bool IsSigned,
 // Enqueue kernel, kernel query, pipe and address space cast built-ins
 // are not mangled.
 bool isNonMangledOCLBuiltin(StringRef Name) {
-  if (!Name.startswith("__"))
+  if (!Name.starts_with("__"))
     return false;
 
   return isEnqueueKernelBI(Name) || isKernelQueryBI(Name) ||
@@ -445,7 +418,7 @@ Op getSPIRVFuncOC(StringRef S, SmallVectorImpl<std::string> *Dec) {
   if (!oclIsBuiltin(S, Name))
     Name = S;
   StringRef R(Name);
-  if ((!Name.startswith(kSPIRVName::Prefix) && !isNonMangledOCLBuiltin(S)) ||
+  if ((!Name.starts_with(kSPIRVName::Prefix) && !isNonMangledOCLBuiltin(S)) ||
       !getByName(dePrefixSPIRVName(R, Postfix).str(), OC)) {
     return OpNop;
   }
@@ -475,13 +448,13 @@ bool oclIsBuiltin(StringRef Name, StringRef &DemangledName, bool IsCpp) {
     DemangledName = Name.drop_front(2);
     return true;
   }
-  if (!Name.startswith("_Z"))
+  if (!Name.starts_with("_Z"))
     return false;
   // OpenCL C++ built-ins are declared in cl namespace.
   // TODO: consider using 'St' abbriviation for cl namespace mangling.
   // Similar to ::std:: in C++.
   if (IsCpp) {
-    if (!Name.startswith("_ZN"))
+    if (!Name.starts_with("_ZN"))
       return false;
     // Skip CV and ref qualifiers.
     size_t NameSpaceStart = Name.find_first_not_of("rVKRO", 3);
@@ -551,11 +524,6 @@ ParamType lastFuncParamType(StringRef MangledName) {
   char Mangled = Copy.back();
   std::string Mangled2 = Copy.substr(Copy.size() - 2);
 
-  std::string Mangled5 = Copy.substr(Copy.size() - 5);
-  if (Mangled5 == "DF16b") {
-    return ParamType::FLOAT;
-  }
-
   if (isMangledTypeFP(Mangled) || isMangledTypeHalf(Mangled2)) {
     return ParamType::FLOAT;
   } else if (isMangledTypeUnsigned(Mangled)) {
@@ -581,11 +549,6 @@ bool containsUnsignedAtomicType(StringRef Name) {
       Name[Loc + strlen(kMangledName::AtomicPrefixIncoming)]);
 }
 
-Constant *castToVoidFuncPtr(Function *F) {
-  auto T = getVoidFuncPtrType(F->getParent());
-  return ConstantExpr::getBitCast(F, T);
-}
-
 bool hasArrayArg(Function *F) {
   for (auto I = F->arg_begin(), E = F->arg_end(); I != E; ++I) {
     LLVM_DEBUG(dbgs() << "[hasArrayArg] " << *I << '\n');
@@ -599,7 +562,7 @@ bool hasArrayArg(Function *F) {
 /// Convert a struct name from the name given to it in Itanium name mangling to
 /// the name given to it as an LLVM opaque struct.
 static std::string demangleBuiltinOpenCLTypeName(StringRef MangledStructName) {
-  assert(MangledStructName.startswith("ocl_") &&
+  assert(MangledStructName.starts_with("ocl_") &&
          "Not a valid builtin OpenCL mangled name");
   // Bare structure type that starts with ocl_ is a builtin opencl type.
   // See clang/lib/CodeGen/CGOpenCLRuntime for how these map to LLVM types
@@ -618,7 +581,7 @@ static std::string demangleBuiltinOpenCLTypeName(StringRef MangledStructName) {
   if (LlvmStructName.empty()) {
     LlvmStructName = "opencl.";
     LlvmStructName += MangledStructName.substr(4); // Strip off ocl_
-    if (!MangledStructName.endswith("_t"))
+    if (!MangledStructName.ends_with("_t"))
       LlvmStructName += "_t";
   }
   return LlvmStructName;
@@ -642,21 +605,6 @@ static Type *parsePrimitiveType(LLVMContext &Ctx, StringRef Name) {
 
 } // namespace SPIRV
 
-namespace {
-
-// Return the value for when the dimension index of a builtin is out of range.
-uint64_t getBuiltinOutOfRangeValue(StringRef VarName) {
-  assert(VarName.starts_with("__spirv_BuiltIn"));
-  return StringSwitch<uint64_t>(VarName)
-      .EndsWith("GlobalSize", 1)
-      .EndsWith("NumWorkgroups", 1)
-      .EndsWith("WorkgroupSize", 1)
-      .EndsWith("EnqueuedWorkgroupSize", 1)
-      .Default(0);
-}
-
-} // anonymous namespace
-
 // The demangler node hierarchy doesn't use LLVM's RTTI helper functions (as it
 // also needs to live in libcxxabi). By specializing this implementation here,
 // we can add support for these functions.
@@ -738,7 +686,7 @@ parseNode(Module *M, const llvm::itanium_demangle::Node *ParamType,
     // pointer element types, the only relevant names are those corresponding
     // to the OpenCL special types (which all begin with "ocl_").
     StringRef Arg(stringify(Name));
-    if (Arg.startswith("ocl_")) {
+    if (Arg.starts_with("ocl_")) {
       const std::string StructName = demangleBuiltinOpenCLTypeName(Arg);
       PointeeTy = GetStructType(StructName);
     } else if (Arg.consume_front("__spirv_")) {
@@ -787,9 +735,9 @@ parseNode(Module *M, const llvm::itanium_demangle::Node *ParamType,
           StructName += NameSuffixPair.second;
         }
         PointeeTy = GetStructType(StructName);
-      } else if (MangledStructName.startswith("opencl.")) {
+      } else if (MangledStructName.starts_with("opencl.")) {
         PointeeTy = GetStructType(MangledStructName);
-      } else if (MangledStructName.startswith("ocl_")) {
+      } else if (MangledStructName.starts_with("ocl_")) {
         const std::string StructName =
             demangleBuiltinOpenCLTypeName(MangledStructName);
         PointeeTy = TypedPointerType::get(GetStructType(StructName), 0);
@@ -832,8 +780,8 @@ bool getParameterTypes(Function *F, SmallVectorImpl<Type *> &ArgTys,
   // If there's no mangled name, we can't do anything. Also, if there's no
   // parameters, do nothing.
   StringRef Name = F->getName();
-  if (!Name.startswith("_Z") || F->arg_empty())
-    return Name.startswith("_Z");
+  if (!Name.starts_with("_Z") || F->arg_empty())
+    return Name.starts_with("_Z");
 
   Module *M = F->getParent();
   auto GetStructType = [&](StringRef Name) {
@@ -939,6 +887,46 @@ bool getParameterTypes(Function *F, SmallVectorImpl<Type *> &ArgTys,
   return DemangledSuccessfully;
 }
 
+bool getRetParamSignedness(Function *F, ParamSignedness &RetSignedness,
+                           SmallVectorImpl<ParamSignedness> &ArgSignedness) {
+  using namespace llvm::itanium_demangle;
+  StringRef Name = F->getName();
+  if (!Name.starts_with("_Z") || F->arg_empty())
+    return false;
+
+  ManglingParser<DefaultAllocator> Demangler(Name.begin(), Name.end());
+  // If it's not a function name encoding, bail out.
+  auto *RootNode = dyn_cast_or_null<FunctionEncoding>(Demangler.parse());
+  if (!RootNode)
+    return false;
+
+  auto GetSignedness = [](const itanium_demangle::Node *N) {
+    if (!N)
+      return ParamSignedness::Unknown;
+    if (const auto *Vec = dyn_cast<itanium_demangle::VectorType>(N))
+      N = Vec->getBaseType();
+    if (const auto *Name = dyn_cast<NameType>(N)) {
+      StringRef Arg(stringify(Name));
+      if (Arg.starts_with("unsigned"))
+        return ParamSignedness::Unsigned;
+      if (Arg.equals("char") || Arg.equals("short") || Arg.equals("int") ||
+          Arg.equals("long"))
+        return ParamSignedness::Signed;
+    }
+    return ParamSignedness::Unknown;
+  };
+  RetSignedness = GetSignedness(RootNode->getReturnType());
+  ArgSignedness.resize(F->arg_size());
+  for (const auto &[I, ParamType] : llvm::enumerate(RootNode->getParams())) {
+    if (F->getArg(I)->getType()->isIntOrIntVectorTy())
+      ArgSignedness[I] = GetSignedness(ParamType);
+    else
+      ArgSignedness[I] = ParamSignedness::Unknown;
+  }
+
+  return true;
+}
+
 CallInst *mutateCallInst(
     Module *M, CallInst *CI,
     std::function<std::string(CallInst *, std::vector<Value *> &)> ArgMutate,
@@ -952,7 +940,7 @@ CallInst *mutateCallInst(
     InstName = CI->getName().str();
     CI->setName(InstName + ".old");
   }
-  auto NewCI = addCallInst(M, NewName, CI->getType(), Args, Attrs, CI, Mangle,
+  auto *NewCI = addCallInst(M, NewName, CI->getType(), Args, Attrs, CI, Mangle,
                            InstName, TakeFuncName);
   NewCI->setDebugLoc(CI->getDebugLoc());
   LLVM_DEBUG(dbgs() << " => " << *NewCI << '\n');
@@ -973,9 +961,9 @@ Instruction *mutateCallInst(
   Type *RetTy = CI->getType();
   auto NewName = ArgMutate(CI, Args, RetTy);
   StringRef InstName = CI->getName();
-  auto NewCI = addCallInst(M, NewName, RetTy, Args, Attrs, CI, Mangle, InstName,
+  auto *NewCI = addCallInst(M, NewName, RetTy, Args, Attrs, CI, Mangle, InstName,
                            TakeFuncName);
-  auto NewI = RetMutate(NewCI);
+  auto *NewI = RetMutate(NewCI);
   NewI->takeName(CI);
   NewI->setDebugLoc(CI->getDebugLoc());
   LLVM_DEBUG(dbgs() << " => " << *NewI << '\n');
@@ -989,9 +977,9 @@ void mutateFunction(
     Function *F,
     std::function<std::string(CallInst *, std::vector<Value *> &)> ArgMutate,
     BuiltinFuncMangleInfo *Mangle, AttributeList *Attrs, bool TakeFuncName) {
-  auto M = F->getParent();
+  auto *M = F->getParent();
   for (auto I = F->user_begin(), E = F->user_end(); I != E;) {
-    if (auto CI = dyn_cast<CallInst>(*I++))
+    if (auto *CI = dyn_cast<CallInst>(*I++))
       mutateCallInst(M, CI, ArgMutate, Mangle, Attrs, TakeFuncName);
   }
   if (F->use_empty())
@@ -1018,10 +1006,10 @@ CallInst *addCallInst(Module *M, StringRef FuncName, Type *RetTy,
                       Instruction *Pos, BuiltinFuncMangleInfo *Mangle,
                       StringRef InstName, bool TakeFuncName) {
 
-  auto F = getOrCreateFunction(M, RetTy, getTypes(Args), FuncName, Mangle,
+  auto *F = getOrCreateFunction(M, RetTy, getTypes(Args), FuncName, Mangle,
                                Attrs, TakeFuncName);
   // Cannot assign a Name to void typed values
-  auto CI = CallInst::Create(F, Args, RetTy->isVoidTy() ? "" : InstName, Pos);
+  auto *CI = CallInst::Create(F, Args, RetTy->isVoidTy() ? "" : InstName, Pos);
   CI->setCallingConv(F->getCallingConv());
   CI->setAttributes(F->getAttributes());
   return CI;
@@ -1050,7 +1038,7 @@ Value *addVector(Instruction *InsPos, ValueVecRange Range) {
     return *Range.first;
   assert(isValidVectorSize(VecSize) && "Invalid vector size");
   IRBuilder<> Builder(InsPos);
-  auto Vec = Builder.CreateVectorSplat(VecSize, *Range.first);
+  auto *Vec = Builder.CreateVectorSplat(VecSize, *Range.first);
   unsigned Index = 1;
   for (++Range.first; Range.first != Range.second; ++Range.first, ++Index)
     Vec = Builder.CreateInsertElement(
@@ -1061,17 +1049,13 @@ Value *addVector(Instruction *InsPos, ValueVecRange Range) {
 
 void makeVector(Instruction *InsPos, std::vector<Value *> &Ops,
                 ValueVecRange Range) {
-  auto Vec = addVector(InsPos, Range);
+  auto *Vec = addVector(InsPos, Range);
   Ops.erase(Range.first, Range.second);
   Ops.push_back(Vec);
 }
 
-Constant *castToInt8Ptr(Constant *V, unsigned Addr = 0) {
-  return ConstantExpr::getBitCast(V, Type::getInt8PtrTy(V->getContext(), Addr));
-}
-
 PointerType *getInt8PtrTy(PointerType *T) {
-  return Type::getInt8PtrTy(T->getContext(), T->getAddressSpace());
+  return PointerType::get(T->getContext(), T->getAddressSpace());
 }
 
 Value *castToInt8Ptr(Value *V, Instruction *Pos) {
@@ -1084,14 +1068,6 @@ IntegerType *getSizetType(Module *M) {
                                 M->getDataLayout().getPointerSizeInBits(0));
 }
 
-Type *getVoidFuncType(Module *M) {
-  return FunctionType::get(Type::getVoidTy(M->getContext()), false);
-}
-
-Type *getVoidFuncPtrType(Module *M, unsigned AddrSpace) {
-  return PointerType::get(getVoidFuncType(M), AddrSpace);
-}
-
 ConstantInt *getInt64(Module *M, int64_t Value) {
   return ConstantInt::getSigned(Type::getInt64Ty(M->getContext()), Value);
 }
@@ -1197,17 +1173,8 @@ std::tuple<unsigned, unsigned, std::string> getSPIRVSource(Module *M) {
   return Tup;
 }
 
-ConstantInt *mapUInt(Module *M, ConstantInt *I,
-                     std::function<unsigned(unsigned)> F) {
-  return ConstantInt::get(I->getType(), F(I->getZExtValue()), false);
-}
-
-ConstantInt *mapSInt(Module *M, ConstantInt *I, std::function<int(int)> F) {
-  return ConstantInt::get(I->getType(), F(I->getSExtValue()), true);
-}
-
 bool isDecoratedSPIRVFunc(const Function *F, StringRef &UndecoratedName) {
-  if (!F->hasName() || !F->getName().startswith(kSPIRVName::Prefix))
+  if (!F->hasName() || !F->getName().starts_with(kSPIRVName::Prefix))
     return false;
   UndecoratedName = F->getName();
   return true;
@@ -1348,8 +1315,6 @@ static SPIR::RefParamType transTypeDesc(Type *Ty,
     return SPIR::RefParamType(new SPIR::PrimitiveType(SPIR::PRIMITIVE_FLOAT));
   if (Ty->isDoubleTy())
     return SPIR::RefParamType(new SPIR::PrimitiveType(SPIR::PRIMITIVE_DOUBLE));
-  if (Ty->isBFloatTy())
-    return SPIR::RefParamType(new SPIR::PrimitiveType(SPIR::PRIMITIVE_BFLOAT));
   if (auto *VecTy = dyn_cast<FixedVectorType>(Ty)) {
     return SPIR::RefParamType(new SPIR::VectorType(
         transTypeDesc(VecTy->getElementType(), Info), VecTy->getNumElements()));
@@ -1362,9 +1327,9 @@ static SPIR::RefParamType transTypeDesc(Type *Ty,
     auto Name = Ty->getStructName();
     std::string Tmp;
 
-    if (Name.startswith(kLLVMTypeName::StructPrefix))
+    if (Name.starts_with(kLLVMTypeName::StructPrefix))
       Name = Name.drop_front(strlen(kLLVMTypeName::StructPrefix));
-    if (Name.startswith(kSPIRVTypeName::PrefixAndDelim)) {
+    if (Name.starts_with(kSPIRVTypeName::PrefixAndDelim)) {
       Name = Name.substr(sizeof(kSPIRVTypeName::PrefixAndDelim) - 1);
       Tmp = Name.str();
       auto Pos = Tmp.find(kSPIRVTypeName::Delimiter); // first dot
@@ -1416,10 +1381,10 @@ static SPIR::RefParamType transTypeDesc(Type *Ty,
     if (isa<FunctionType>(ET)) {
       assert(isVoidFuncTy(cast<FunctionType>(ET)) && "Not supported");
       EPT = new SPIR::BlockType;
-    } else if (auto StructTy = dyn_cast<StructType>(ET)) {
+    } else if (auto *StructTy = dyn_cast<StructType>(ET)) {
       LLVM_DEBUG(dbgs() << "ptr to struct: " << *Ty << '\n');
       auto TyName = StructTy->getStructName();
-      if (TyName.startswith(kSPR2TypeName::OCLPrefix)) {
+      if (TyName.starts_with(kSPR2TypeName::OCLPrefix)) {
         auto DelimPos = TyName.find_first_of(kSPR2TypeName::Delimiter,
                                              strlen(kSPR2TypeName::OCLPrefix));
         if (DelimPos != StringRef::npos)
@@ -1430,13 +1395,13 @@ static SPIR::RefParamType transTypeDesc(Type *Ty,
       auto Prim = getOCLTypePrimitiveEnum(TyName);
       if (StructTy->isOpaque()) {
         if (TyName == "opencl.block") {
-          auto BlockTy = new SPIR::BlockType;
+          auto *BlockTy = new SPIR::BlockType;
           // Handle block with local memory arguments according to OpenCL 2.0
           // spec.
           if (Info.IsLocalArgBlock) {
             SPIR::RefParamType VoidTyRef(
                 new SPIR::PrimitiveType(SPIR::PRIMITIVE_VOID));
-            auto VoidPtrTy = new SPIR::PointerType(VoidTyRef);
+            auto *VoidPtrTy = new SPIR::PointerType(VoidTyRef);
             VoidPtrTy->setAddressSpace(SPIR::ATTR_LOCAL);
             // "__local void *"
             BlockTy->setParam(0, SPIR::RefParamType(VoidPtrTy));
@@ -1449,7 +1414,7 @@ static SPIR::RefParamType transTypeDesc(Type *Ty,
           if (Prim == SPIR::PRIMITIVE_PIPE_RO_T ||
               Prim == SPIR::PRIMITIVE_PIPE_WO_T) {
             SPIR::RefParamType OpaqueTyRef(new SPIR::PrimitiveType(Prim));
-            auto OpaquePtrTy = new SPIR::PointerType(OpaqueTyRef);
+            auto *OpaquePtrTy = new SPIR::PointerType(OpaqueTyRef);
             OpaquePtrTy->setAddressSpace(getOCLOpaqueTypeAddrSpace(Prim));
             EPT = OpaquePtrTy;
           } else {
@@ -1505,9 +1470,9 @@ Value *getScalarOrArray(Value *V, unsigned Size, Instruction *Pos) {
 }
 
 Constant *getScalarOrVectorConstantInt(Type *T, uint64_t V, bool IsSigned) {
-  if (auto IT = dyn_cast<IntegerType>(T))
+  if (auto *IT = dyn_cast<IntegerType>(T))
     return ConstantInt::get(IT, V);
-  if (auto VT = dyn_cast<FixedVectorType>(T)) {
+  if (auto *VT = dyn_cast<FixedVectorType>(T)) {
     std::vector<Constant *> EV(
         VT->getNumElements(),
         getScalarOrVectorConstantInt(VT->getElementType(), V, IsSigned));
@@ -1519,7 +1484,7 @@ Constant *getScalarOrVectorConstantInt(Type *T, uint64_t V, bool IsSigned) {
 
 Value *getScalarOrArrayConstantInt(Instruction *Pos, Type *T, unsigned Len,
                                    uint64_t V, bool IsSigned) {
-  if (auto IT = dyn_cast<IntegerType>(T)) {
+  if (auto *IT = dyn_cast<IntegerType>(T)) {
     assert(Len == 1 && "Invalid length");
     return ConstantInt::get(IT, V, IsSigned);
   }
@@ -1527,23 +1492,23 @@ Value *getScalarOrArrayConstantInt(Instruction *Pos, Type *T, unsigned Len,
     unsigned PointerSize =
         Pos->getModule()->getDataLayout().getPointerTypeSizeInBits(T);
     auto *ET = Type::getIntNTy(T->getContext(), PointerSize);
-    auto AT = ArrayType::get(ET, Len);
+    auto *AT = ArrayType::get(ET, Len);
     std::vector<Constant *> EV(Len, ConstantInt::get(ET, V, IsSigned));
-    auto CA = ConstantArray::get(AT, EV);
-    auto Alloca = new AllocaInst(AT, 0, "", Pos);
+    auto *CA = ConstantArray::get(AT, EV);
+    auto *Alloca = new AllocaInst(AT, 0, "", Pos);
     new StoreInst(CA, Alloca, Pos);
-    auto Zero = ConstantInt::getNullValue(Type::getInt32Ty(T->getContext()));
+    auto *Zero = ConstantInt::getNullValue(Type::getInt32Ty(T->getContext()));
     Value *Index[] = {Zero, Zero};
     auto *Ret = GetElementPtrInst::CreateInBounds(AT, Alloca, Index, "", Pos);
     LLVM_DEBUG(dbgs() << "[getScalarOrArrayConstantInt] Alloca: " << *Alloca
                       << ", Return: " << *Ret << '\n');
     return Ret;
   }
-  if (auto AT = dyn_cast<ArrayType>(T)) {
-    auto ET = AT->getArrayElementType();
+  if (auto *AT = dyn_cast<ArrayType>(T)) {
+    auto *ET = AT->getArrayElementType();
     assert(AT->getArrayNumElements() == Len);
     std::vector<Constant *> EV(Len, ConstantInt::get(ET, V, IsSigned));
-    auto Ret = ConstantArray::get(AT, EV);
+    auto *Ret = ConstantArray::get(AT, EV);
     LLVM_DEBUG(dbgs() << "[getScalarOrArrayConstantInt] Array type: " << *AT
                       << ", Return: " << *Ret << '\n');
     return Ret;
@@ -1576,35 +1541,6 @@ bool isSPIRVConstantName(StringRef TyName) {
   return false;
 }
 
-std::string getSPIRVImageSampledTypeName(SPIRVType *Ty) {
-  switch (Ty->getOpCode()) {
-  case OpTypeVoid:
-    return kSPIRVImageSampledTypeName::Void;
-  case OpTypeInt:
-    if (Ty->getIntegerBitWidth() == 32) {
-      if (static_cast<SPIRVTypeInt *>(Ty)->isSigned())
-        return kSPIRVImageSampledTypeName::Int;
-      else
-        return kSPIRVImageSampledTypeName::UInt;
-    }
-    break;
-  case OpTypeFloat:
-    switch (Ty->getFloatBitWidth()) {
-    case 16:
-      return kSPIRVImageSampledTypeName::Half;
-    case 32:
-      return kSPIRVImageSampledTypeName::Float;
-    default:
-      break;
-    }
-    break;
-  default:
-    break;
-  }
-  llvm_unreachable("Invalid sampled type for image");
-  return std::string();
-}
-
 // ToDo: Find a way to represent uint sampled type in LLVM, maybe an
 //      opaque type.
 Type *getLLVMTypeForSPIRVImageSampledTypePostfix(StringRef Postfix,
@@ -1659,7 +1595,7 @@ std::string getImageBaseTypeName(StringRef Name) {
   SmallVector<StringRef, 4> SubStrs;
   const char Delims[] = {kSPR2TypeName::Delimiter, 0};
   Name.split(SubStrs, Delims);
-  if (Name.startswith(kSPR2TypeName::OCLPrefix)) {
+  if (Name.starts_with(kSPR2TypeName::OCLPrefix)) {
     Name = SubStrs[1];
   } else {
     Name = SubStrs[0];
@@ -1707,8 +1643,8 @@ bool eraseIfNoUse(Function *F) {
 
   dumpUsers(F, "[eraseIfNoUse] ");
   for (auto UI = F->user_begin(), UE = F->user_end(); UI != UE;) {
-    auto U = *UI++;
-    if (auto CE = dyn_cast<ConstantExpr>(U)) {
+    auto *U = *UI++;
+    if (auto *CE = dyn_cast<ConstantExpr>(U)) {
       if (CE->use_empty()) {
         CE->dropAllReferences();
         Changed = true;
@@ -1723,20 +1659,6 @@ bool eraseIfNoUse(Function *F) {
   return Changed;
 }
 
-void eraseIfNoUse(Value *V) {
-  if (!V->use_empty())
-    return;
-  if (Constant *C = dyn_cast<Constant>(V)) {
-    C->destroyConstant();
-    return;
-  }
-  if (Instruction *I = dyn_cast<Instruction>(V)) {
-    if (!I->mayHaveSideEffects())
-      I->eraseFromParent();
-  }
-  eraseIfNoUse(dyn_cast<Function>(V));
-}
-
 bool eraseUselessFunctions(Module *M) {
   bool Changed = false;
   for (auto I = M->begin(), E = M->end(); I != E;)
@@ -1783,7 +1705,7 @@ std::string mangleBuiltin(StringRef UniqName, ArrayRef<Type *> ArgTypes,
     for (unsigned I = 0, E = BIVarArgNegative ? ArgTypes.size()
                                               : (unsigned)BtnInfo->getVarArg();
          I != E; ++I) {
-      auto T = ArgTypes[I];
+      auto *T = ArgTypes[I];
       auto MangleInfo = BtnInfo->getTypeMangleInfo(I);
       if (MangleInfo.PointerTy && T->isPointerTy()) {
         T = MangleInfo.PointerTy;
@@ -1867,7 +1789,7 @@ bool isSPIRVOCLExtInst(const CallInst *CI, OCLExtOpKind *ExtOp) {
   if (!oclIsBuiltin(CI->getCalledFunction()->getName(), DemangledName))
     return false;
   StringRef S = DemangledName;
-  if (!S.startswith(kSPIRVName::Prefix))
+  if (!S.starts_with(kSPIRVName::Prefix))
     return false;
   S = S.drop_front(strlen(kSPIRVName::Prefix));
   auto Loc = S.find(kSPIRVPostfix::Divider);
@@ -1950,9 +1872,6 @@ bool checkTypeForSPIRVExtendedInstLowering(IntrinsicInst *II, SPIRVModule *BM) {
       NumElems = VecTy->getNumElements();
       Ty = VecTy->getElementType();
     }
-    if (Ty->isBFloatTy() &&
-        BM->hasCapability(internal::CapabilityBFloat16ArithmeticINTEL))
-      return true;
     if ((!Ty->isFloatTy() && !Ty->isDoubleTy() && !Ty->isHalfTy()) ||
         (!BM->hasCapability(CapabilityVectorAnyINTEL) &&
          ((NumElems > 4) && (NumElems != 8) && (NumElems != 16)))) {
@@ -1969,9 +1888,6 @@ bool checkTypeForSPIRVExtendedInstLowering(IntrinsicInst *II, SPIRVModule *BM) {
       NumElems = VecTy->getNumElements();
       Ty = VecTy->getElementType();
     }
-    if (Ty->isBFloatTy() &&
-        BM->hasCapability(internal::CapabilityBFloat16ArithmeticINTEL))
-      return true;
     if ((!Ty->isIntegerTy()) ||
         (!BM->hasCapability(CapabilityVectorAnyINTEL) &&
          ((NumElems > 4) && (NumElems != 8) && (NumElems != 16)))) {
@@ -2220,50 +2136,22 @@ bool lowerBuiltinCallsToVariables(Module *M) {
     bool IsVec = F.getFunctionType()->getNumParams() > 0;
     Type *GVType =
         IsVec ? FixedVectorType::get(F.getReturnType(), 3) : F.getReturnType();
-    GlobalVariable *BV = nullptr;
-    // Consider the following LLVM IR:
-    // @__spirv_BuiltInLocalInvocationId = <Global constant>
-    // .....
-    // define spir_kernel void @kernel1(....) {
-    //   %3 = tail call i64 @_Z12get_local_idj(i32 0)
-    //   .....
-    //   return void
-    // }
-    // During the OCLToSPIRV pass, the opencl call will get lowered to
-    // yet another global variable with the name
-    // '@__spirv_BuiltInLocalInvocationId'. In such a case, we would want to
-    // create only a single global variable with this name.
-    if (GlobalVariable *GV = M->getGlobalVariable(BuiltinVarName))
-      BV = GV;
-    else
-      BV = new GlobalVariable(*M, GVType, /*isConstant=*/true,
-                              GlobalValue::ExternalLinkage, nullptr,
-                              BuiltinVarName, 0, GlobalVariable::NotThreadLocal,
-                              SPIRAS_Input);
+    auto *BV = new GlobalVariable(
+        *M, GVType, /*isConstant=*/true, GlobalValue::ExternalLinkage, nullptr,
+        BuiltinVarName, 0, GlobalVariable::NotThreadLocal, SPIRAS_Input);
     for (auto *U : F.users()) {
       auto *CI = dyn_cast<CallInst>(U);
       assert(CI && "invalid instruction");
-      IRBuilder<> Builder(CI);
-      Value *NewValue = Builder.CreateLoad(GVType, BV);
+      const DebugLoc &DLoc = CI->getDebugLoc();
+      Instruction *NewValue = new LoadInst(GVType, BV, "", CI);
+      if (DLoc)
+        NewValue->setDebugLoc(DLoc);
       LLVM_DEBUG(dbgs() << "Transform: " << *CI << " => " << *NewValue << '\n');
       if (IsVec) {
-        auto *GVVecTy = cast<FixedVectorType>(GVType);
-        ConstantInt *Bound = Builder.getInt32(GVVecTy->getNumElements());
-        // Create a select on the index first, to avoid undefined behaviour
-        // due to exceeding the vector size by the extractelement.
-        Value *IndexCmp = Builder.CreateICmpULT(CI->getArgOperand(0), Bound);
-        Constant *ZeroIndex =
-            ConstantInt::get(CI->getArgOperand(0)->getType(), 0);
-        Value *ExtractIndex =
-            Builder.CreateSelect(IndexCmp, CI->getArgOperand(0), ZeroIndex);
-
-        // Extract from builtin variable.
-        NewValue = Builder.CreateExtractElement(NewValue, ExtractIndex);
-
-        // Clamp to out-of-range value.
-        Constant *OutOfRangeVal = ConstantInt::get(
-            F.getReturnType(), getBuiltinOutOfRangeValue(BuiltinVarName));
-        NewValue = Builder.CreateSelect(IndexCmp, NewValue, OutOfRangeVal);
+        NewValue =
+            ExtractElementInst::Create(NewValue, CI->getArgOperand(0), "", CI);
+        if (DLoc)
+          NewValue->setDebugLoc(DLoc);
         LLVM_DEBUG(dbgs() << *NewValue << '\n');
       }
       NewValue->takeName(CI);
@@ -2675,13 +2563,4 @@ std::string getSPIRVFriendlyIRFunctionName(const std::string &UniqName,
   return mangleBuiltin(UniqName, ArgTys, &MangleInfo);
 }
 
-template <typename T>
-MetadataAsValue *map2MDString(LLVMContext &C, SPIRVValue *V) {
-  if (V->getOpCode() != OpConstant)
-    return nullptr;
-  uint64_t Const = static_cast<SPIRVConstant *>(V)->getZExtIntValue();
-  std::string Str = SPIRVMap<T, std::string>::map(static_cast<T>(Const));
-  return MetadataAsValue::get(C, MDString::get(C, Str));
-}
-
 } // namespace SPIRV
